function [coef, leaders, nj] = DxLx1d_bayes_dwt_(data, Nwt, gamint, Jmax)
%% function [coef, leaders, nj] = DWTc2Bayes(data, Nwt, gamint, Jmax)
if nargin<3; gamint=0; Jmax=Inf; end;
if nargin<4; Jmax=Inf; end;
if nargout==1; doLeaders=0; else; doLeaders=1; end
Norm=1;
p=Inf;
%keyboard
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DWT
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-- Initialize the wavelet filters
n = size(data,2) ;  % data length
u = rlistcoefdaub(Nwt) ; nu = 0 ;
[v,nv] = tildeurflippeur(u,nu);  nv = nv + 1;
nls=length(v)/2+1; doff=-1;
[hh1,nh1] = flippeur(u,nu) ; nl=length(hh1);
[gg1,ng1] = flippeur(v,nv) ;

%--- Predict the max # of octaves available given Nwt, and take the min with
nbvoies= fix( log2(n) );
nbvoies = min( Jmax  , nbvoies);  %   safer, casadestime having problems
nbvoies = min( fix(log2(n/(nl+1)))  , nbvoies);  %   safer, casadestime having problems
nbvoies=min(nbvoies,Jmax);
%--- Compute the WT, calculate statistics
njtemp = n;    %  this is the the appro, changes at each scale
approW=data;
approL=data;

for j=1:nbvoies         % Loop Scales
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% PHASE 1: COEFFICIENTS, LEADERS
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %-- Phase 1a: get the wavelet coefficients at this scale
    njtemp = size(approW,2) ;
%     %conv_gg1 = conv(approW,gg1) ;
%     conv_gg1 = LineWiseConv(approW,gg1) ;
%     %conv_hh1 = conv(approW,hh1) ;
%     conv_hh1 = LineWiseConv(approW,hh1) ;
    [conv_gg1,conv_hh1] = LineWiseConv(approW,gg1,hh1);
    approW = conv_hh1(:,nls:2:njtemp+nls-1) ; %-- prepare for Phase 1 at next coarser scale
    decime = conv_gg1(:,nls:2:njtemp+nls-1) ;   % details at scale j. decime always becomes empty near the end,
    nj.W(j) = size(decime,2);            % number of coefficients
    if ~doLeaders
        coef(j).nj=nj.W(j);
    end
    AbsdqkW = abs(decime)*2^(j/2)/2^(j/Norm);   %%%%%%% passage Norme L1
    % fractional integration
    AbsdqkW = AbsdqkW*2^(gamint*j);
    coef(j).value_sbord=(AbsdqkW)*2^(j/2)/2^(j/Norm);
    coef(j).nbord(1)=nls+doff; coef(j).nbord(2)=size(AbsdqkW,2)-nls+2;
    %keyboard
    coef(j).value=AbsdqkW(:,coef(j).nbord(1):coef(j).nbord(2));
    % hmin before integration
    coef(j).supcoefnoint=max(coef(j).value*2^(-gamint*j),[],2);
    [coef(j).supcoef, coef(j).supcoefid]=max(coef(j).value,[],2);
    coef(j).gamma=gamint;
    coef(j).sign=sign(decime);
    nj.W(j) = size(coef(j).value,2);
    %% <---- ADDED 16/04/2007
    
    if doLeaders
        %keyboard
        %-- Phase 1b: calculate the (p-) wavelet leaders at this scale
        if j==1
            decime = abs(conv_gg1(:,nls:2:njtemp+nls-1) ) *2^(j/2)/2^(j/Norm); %%%%%%% passage Norme L1
            
            % fractional integration
            decime = decime*2^(gamint*j);
            leaders(1).sans_voisin.value_sbord = decime;
            tpdec(:,:,1) = decime(:,1:end-2);
            tpdec(:,:,2) = decime(:,2:end-1);
            tpdec(:,:,3) = decime(:,3:end);
            leaders(1).value_sbord = max(tpdec,[],3); clear tpdec
        else
            decime = abs(conv_gg1(:,nls:2:njtemp+nls-1) ) *2^(j/2)/2^(j/Norm) ; %%%%%%% passage Norme L1
            % fractional integration
            decime = decime*2^(gamint*j);
            % fix problem with uneven number of leaders
            %length_detail = 2*size(decime,2);
            length_detail = min(2*size(decime,2), 2*floor(size(leaders(j-1).sans_voisin.value_sbord,2)/2));
            % leaders
            tpdec.sans_voisin(:,:,1) = decime(:,1:length_detail/2);
            tpdec.sans_voisin(:,:,2) = leaders(j-1).sans_voisin.value_sbord(:,1:2:length_detail);
            tpdec.sans_voisin(:,:,3) = leaders(j-1).sans_voisin.value_sbord(:,2:2:length_detail);
            leaders(j).sans_voisin.value_sbord = max(tpdec.sans_voisin,[],3);
            tpdec.value_sbord(:,:,1) = leaders(j).sans_voisin.value_sbord(:,1:end-2);
            tpdec.value_sbord(:,:,2) = leaders(j).sans_voisin.value_sbord(:,2:end-1);
            tpdec.value_sbord(:,:,3) = leaders(j).sans_voisin.value_sbord(:,3:end);
            leaders(j).value_sbord = max(tpdec.value_sbord,[],3); clear tpdec
        end
        leaders(j).value_sbord = [leaders(j).value_sbord(:,1), leaders(j).value_sbord, leaders(j).value_sbord(:,end)];
        leaders(j).nbord(1)=coef(j).nbord(1)+1; leaders(j).nbord(2)=coef(j).nbord(2)-1;
        leaders(j).sans_voisin.value=leaders(j).sans_voisin.value_sbord(:,coef(j).nbord(1):coef(j).nbord(2));
        leaders(j).value=leaders(j).value_sbord(:,leaders(j).nbord(1):leaders(j).nbord(2));
        nj.L(j) = size(leaders(j).value,2);
        
        leaders(j).p=p;
        leaders(j).gamma=gamint;
        [leaders(j).mincoef, leaders(j).mincoefid]=min(leaders(j).value,[],2);
        [leaders(j).supcoefL, leaders(j).supcoefidL]=max(leaders(j).value,[],2);
        leaders(j).supcoefnoint=coef(j).supcoefnoint; leaders(j).supcoef=coef(j).supcoef; leaders(j).supcoefid=coef(j).supcoefid;
        coef(j).mincoef=leaders(j).mincoef; coef(j).mincoefid=leaders(j).mincoefid;
        coef(j).supcoefL=leaders(j).supcoefL; coef(j).supcoefidL=leaders(j).supcoefidL;
    end
end




%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% INLINE FUNCTIONS
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     function [a,b]=convoluteur(c,d,e,f);
%         a=conv(c,e);
%         b=d+f;
%     end

    function [a,b]=flippeur(c,d);
        ls=max(size(c));
        b=-ls+1-d;
        a=fliplr(c);
    end

%     function [a,b]=upsample1(c,d);
%         ls=max(size(c));
%         a=zeros(1,2*ls-1);
%         a(1:2:2*ls-1)=c;
%         b=2*d;
%     end

    function [a,b]=tildeurflippeur(c,d);
        ls=max(size(c));
        k=d:1:ls+d-1;
        b=-ls+1-d;
        a=(-1).^(k).*c;
        a=fliplr(a);
    end

    function h=rlistcoefdaub(regu);
        
        if ~(regu<11 & regu>0),
            mess=[' Ondelette non implantee'];
            disp(mess);
            return;
        else
            if regu==1,
                h=[1/sqrt(2) 1/sqrt(2)];
                %  g=[1/sqrt(2) -1/sqrt(2)];
            end
            if regu==2,
                h(1:2)=[0.482962913145 0.836516303738];
                h(3:4)=[0.224143868042 -0.129409522551];
            end
            if regu==3,
                h(1:2)=[0.332670552950 0.806891509311];
                h(3:4)=[0.459877502118 -0.135011020010];
                h(5:6)=[-0.085441273882 0.035226291882];
            end
            if regu==4,
                h(1:2)=[0.230377813309 0.714846570553];
                h(3:4)=[0.630880767930 -0.027983769417];
                h(5:6)=[-0.187034811719 0.030841381836];
                h(7:8)=[0.032883011667 -0.010597401785];
            end
            if regu==5,
                h(1:2)=[0.160102397974 0.603829269797];
                h(3:4)=[0.724308528438 0.138428145901];
                h(5:6)=[-0.242294887066 -0.032244869585];
                h(7:8)=[0.077571493840 -0.006241490213];
                h(9:10)=[-0.012580751999 0.003335725285];
            end
            if regu==6,
                h(1:2)=[0.111540743350 0.494623890398];
                h(3:4)=[0.751133908021 0.315250351709];
                h(5:6)=[-0.226264693965 -0.129766867567];
                h(7:8)=[0.097501605587 0.027522865530];
                h(9:10)=[-0.031582039318 0.000553842201];
                h(11:12)=[0.004777257511 -0.001077301085];
            end
            if regu==7,
                h(1:2)=[0.077852054085 0.396539319482];
                h(3:4)=[0.729132090846 0.469782287405];
                h(5:6)=[-0.143906003929 -0.224036184994];
                h(7:8)=[0.071309219267 0.080612609151];
                h(9:10)=[-0.038029936935 -0.016574541631];
                h(11:12)=[0.012550998556 0.000429577973];
                h(13:14)=[-0.001801640704 0.000353713800];
            end
            if regu==8,
                h(1:2)=[0.054415842243 0.312871590914];
                h(3:4)=[0.675630736297 0.585354683654];
                h(5:6)=[-0.015829105256 -0.284015542962];
                h(7:8)=[0.000472484574 0.128747426620];
                h(9:10)=[-0.017369301002 -0.044088253931];
                h(11:12)=[0.013981027917 0.008746094047];
                h(13:14)=[-0.004870352993 -0.000391740373];
                h(15:16)=[0.000675449406 -0.000117476784];
            end
            if regu==9,
                h(1:2)=[0.038077947364 0.243834674613];
                h(3:4)=[0.604823123690 0.657288078051];
                h(5:6)=[0.133197385825 -0.293273783279];
                h(7:8)=[-0.096840783223 0.148540749338];
                h(9:10)=[0.030725681479 -0.067632829061];
                h(11:12)=[0.000250947115 0.022361662124];
                h(13:14)=[-0.004723204758 -0.004281503682];
                h(15:16)=[0.001847646883 0.000230385764];
                h(17:18)=[-0.000251963189 0.000039347320];
            end
            if regu==10,
                h(1:2)=[0.026670057901 0.188176800078];
                h(3:4)=[0.52720118932 0.688459039454];
                h(5:6)=[0.281172343661 -0.249846424327];
                h(7:8)=[-0.195946274377 0.127369340336];
                h(9:10)=[0.093057364604 -0.071394147166];
                h(11:12)=[-0.029457536822 0.033212674059];
                h(13:14)=[0.003606553567 -0.010733175483];
                h(15:16)=[0.001395351747 0.001992405295];
                h(17:18)=[-0.000685856695 -0.000116466855];
                h(19:20)=[0.000093588670 -0.000013264203];
            end
        end
    end


%     function [C1,C2] = LineWiseConv(a,b1,b2)
%         
%         nL = size(a,1); L = size(a,2); nb = length(b1);
%         %keyboard
%         b1 = [b1, zeros(1,L-nb)];
%         b1 = [b1,zeros(size(b1))]; B1 = (fft(b1));
%         
%         b2 = [b2, zeros(1,L-nb)];
%         b2 = [b2,zeros(size(b2))]; B2 = (fft(b2));
% 
%         
%         a = [a,zeros(size(a))]; A = fft(a,[],2);
% 
% 
%         C1 = ifft(bsxfun(@times,A,B1),[],2);C1 = C1(:,1:L+nb-1);
%         C2 = ifft(bsxfun(@times,A,B2),[],2);C2 = C2(:,1:L+nb-1);
%         
%         
%         
% 
%     end


  function [C1,C2] = LineWiseConv(a,b1,b2)
%         keyboard
%         tic
%         nL = size(a,1); na = size(a,2); nb = length(b1);
%         nT = na + nb;
%         %keyboard
%         tb1 = zeros(1,nT); tb1(1:nb)=b1;B1 = (fft(tb1));
%         tb2 = zeros(1,nT); tb2(1:nb)=b2;B2 = (fft(tb2));
%         ta  = zeros(nL,nT); ta(:,1:na) = a; A = fft(ta,[],2);
% %         keyboard
% 
% 
%         C1 = ifft(bsxfun(@times,A,B1),[],2);C1 = C1(:,1:end-1);
%         C2 = ifft(bsxfun(@times,A,B2),[],2);C2 = C2(:,1:end-1);
% %         toc
        
%         tic
        C1=conv2(a,b1);
        C2=conv2(a,b2);
%         toc

    end

end % function end
