function [I,Ni,Nj,wr,ind,VV,mV,Lx] = pDWT_2DNZ(im3D,j1,j2,Nwt,eta,gamint)
%function [I,Ni,Nj,wr,ind,VV] = PrepPerioImage3D_Complex(im3D,j1,j2,Nwt,eta,gamint)
% Compute Fourier coefficients for the Whittle approximation
%
%
% OUTPUT
%   I.Y     Fourier coefficients of the log-leaders
%   I.Y     periodogram of the log-leaders
%   Ni      not used
%   Nj      number of log-leaders
%   wr      frequencies used in the Whittle approximation
%   ind     index of the frequencies used in the Whittle approximation
%   VV      scale-wise variance of the log-leaders

if nargin < 6
    gamint = 0;
end
M = size(im3D,3);

for t=1:M

        tmp=im3D(:,:,t);
        if sum(~isfinite(tmp(:)))
            %keyboard
            for j=j1:j2;
                leaders(j).matrix=inf(size(im3D(1:2^j:end,1:2^j:end,t)));
            end
        else
            % Wavelet transform
            [coef, leaders, nj] = DxLx2d_bayes_dwt(im3D(:,:,t),Nwt,j2,gamint);%Nj = sqrt(nj.L);
            if length(leaders)<j2 & ~sum(isfinite(leaders(end).allmatrix(:)))
                for j=length(leaders)+1:j2; leaders(j).allmatrix=leaders(j-1).allmatrix(1:2:end,1:2:end); end
                %keyboard
            end
        end
        
        if length(leaders)<j2
            fprintf('\n\n ERROR:\n There are less than j2 scales available.\n Reduce j2 and restart.\n\n');
            error(' ');
        end

%     % Wavelet transform
%     [coef, leaders, nj] = DxLx2d_bayes_dwt(im3D(:,:,t),Nwt,j2,gamint);%Nj = sqrt(nj.L);
    
%     % Wavelet transform
%     [coef, leaders, nj] = DxLx2d_bayes_dwt(im3D(:,:,t),Nwt,j2,gamint);%Nj = sqrt(nj.L);
%     if length(leaders)<j2 & ~sum(isfinite(leaders(end).allmatrix(:)))
%         for j=length(leaders)+1:j2; leaders(j).allmatrix=leaders(j-1).allmatrix(1:2:end,1:2:end); end
%         %keyboard
%     end
    
    for j=j1:j2;
        lx{j}   = log(leaders(j).matrix);
        Lx{j}{t} = lx{j};
        VV(t,j) = var(lx{j}(:));
        mV(t,j) = mean(lx{j}(:));
        Nj(j,:) = size(lx{j});
    end
    
    if t==1;[wr,ind] = fft_freqNZ(j1,j2,Nj,eta);end
    
    
    Pv = [];Yv = []; Ni = [];
    for j=j1:j2
        %Y = ((fftshift(fft2(lx{j}-mean(lx{j}(:))))))/sqrt(prod(Nj(j,:)));
        Y = ((fftshift(fft2(lx{j}))))/sqrt(prod(Nj(j,:)));
        P = abs(Y).^2;
        % HW 18/03/2017: fixed missing data issue
        %keyboard
        try
            tp = P(ind{j});Pv = [Pv tp'];
        catch
            %tp=nan(size(ind{j}));
            tp=nan(sum(ind{j}(:)),1);Pv = [Pv tp'];
        end
        try
            tp = Y(ind{j});Yv = [Yv tp'];
        catch
            tp=nan(sum(ind{j}(:)),1);Yv = [Yv tp'];
        end
        
        
    end
    
    if t==1;
        Ni = ones(size(Yv));
    end
    
    I.Y(t,:)=Yv;
    I.P(t,:)=Pv;
    
end
end










