function [wB] = psamplewB(y0,dD,X,w0,dS)
%keyboard
% iN = isnan(y0(:,1));
% y0(iN,:)=0;
% Parallel sampling of  2 x M vectors wB, where each wB(:,m)
% is distributed according to the Gaussian conditional posterior
% N( (X'*D^-1*X + S^-1)^-1*(X'*D^-1*y0 + S^-1*w0) , (X'D^-1+S^-1)^-1 )
%  induced by
% - a Gaussian likelihood exp(-0.5*(y0-Xw)'*D^-1*(y0-Xw))
% - a Gaussian prior exp(-0.5*(w-w0)*S^-1*(w-w0))
% with
% - y0 a N x M
% - D a N x N diagonal matrix diag(dD)
% - X a 2 x 2 design matrix
% - w0 a 2 x 1 mean vector
% - S a 2 x 2 !! diagonal !! covariance matrix
%
% Notations
% - A = X'*D^-1*X
% - B = (X'*D^-1*X+S^-1)^-1
% - C Cholesky decomposition of B
% - mwB = (X'*D^-1*X + S^-1)^-1*(X'*D^-1*y0 + S^-1*w0)

% keyboard
% N = 3; M = 1;
% dS = 0.001*[1;2]; S = diag(dS);
% dD = rand(M,N); D = diag(dD);
% y0 = randn(M,N);
% w0 = randn(2,M);
%
% X = [1:N;1*ones(1,N)]';

M = size(w0,2);
LL = randn(2,M);

%keyboard
if 1
    dS1 = dS(1,:)';dS2= dS(2,:)';
    
    
    Xi1 = X(:,1); Xi2 = X(:,2);
    idD = 1./dD;
    
    % Parallel computation of A = X'*D^-1*X
    A11 = idD*(Xi1.^2);
    A12 = idD*(Xi1.*Xi2);
    A22 = idD*(Xi2.^2);
    
    % Parallel computation of B = (X'*D^-1*X+S^-1)^-1
    detiB   = (A11+1./dS1).*(A22+1./dS2)-A12.^2;
    B11 = (A22 + 1./dS2)./detiB;
    B22 = (A11 + 1./dS1)./detiB;
    B12 = -A12./detiB;
    
    % Parallel computation of C the cholesky decomposition of B
    C11 = sqrt(B11);
    C12 = B12./C11;
    C22 = sqrt(B22-C12.^2);
    %keyboard
    % Parallel computation of  mwB = (X'*D^-1*X + S^-1)^-1*(X'*D^-1*y0' + S^-1*w0)
    tmwB = bsxfun(@plus,bsxfun(@times,1./dS,w0),X'*(idD.*y0)');
    mwB(1,:)  = B11'.*tmwB(1,:)+B12'.*tmwB(2,:);
    mwB(2,:)  = B12'.*tmwB(1,:)+B22'.*tmwB(2,:);
    
    L1 = LL(1,:); L2 = LL(2,:);
    
    wB(1,:) = mwB(1,:)+L1.*C11';
    wB(2,:) = mwB(2,:)+L1.*C12'+L2.*C22';
    
    %[mean(C11) mean(C12) mean(C22)]
else
    for k=1:M
        S = diag([dS(1),dS(2)]);
        D = diag(dD(k,:));
        SS = (X'*D^-1*X+S^-1)^-1;
        mwB(:,k) = SS*(X'*D^-1*y0(k,:)' + S^-1*w0(:,k));
        wB(:,k) = tmwB(:,k) + chol(SS)'*LL(:,k);
    end
    
end

%keyboard
end
