%% demo_TS_2Dplane.m
% tutorial for the joint estimation of (c1,c2) for a collection of time series
% registered on a planar array of sensors

clear all; clc;

% Load tools
pth=which('est_bayes_mvmfa');if ispc; ii=strfind(pth,'\');else;ii=strfind(pth,'/');end;pth=pth(1:ii(end));addpath(genpath(pth));
%currentpath = cd;addpath([currentpath(1:end-5)]);addpath([currentpath(1:end-5) '/all_tools/']);

%% params
NII=1;     % number of integral scales (default: 1)
Nwt = 2;    % vanishing moments of wavelet
gamint = 0; % fractional integration parameter for wavelet leaders
j1=1;       % scaling range, initialized below for different data sets
j2=2;       % scaling range, initialized below for different data sets

Nbi = 300;  % burn-in of MCMC
Nmc = 1000;  % length of MCMC


%% SYNTHETIC DATA SET

load('MRWseries640_6.mat'); C1T=C1;C2T=C2;C1 = [0.5 0.9]; C2 = [-0.05 0]; % for figure
SL=3;
SL=5;
data=squeeze(data(:,:,SL));
figure(1);clf;set(gcf,'position',[10 500 500 400]);
imagesc(data);colormap bone;
title('A single image with two distinct zones of multifractality');
M2=MASK; M2(MASK==1)=C2T(1);M2(MASK==2)=C2T(2);
figure(5);clf;hold on;set(gcf,'position',[600 500 500 400]);
imagesc(flipud(M2)); set(gca,'clim',C2); axis tight; set(gca,'ytick',[100:100:600]-60);set(gca,'yticklabel',[600:-100:100]);
title('Corresponding mask for values for c2');
        
disp('The data set...'); drawnow;pause(1);

%% patch-wise analysis
disp('The data set cut in non-overlapping patches...'); 
L = 2^6;
figure(1);hold on; for n1=1:9;plot(L*n1*[1 1],[0 640],'r');plot([0 640],L*n1*[1 1],'r');end
figure(5);hold on; for n1=1:9;plot(L*n1*[1 1],[0 640],'r');plot([0 640],L*n1*[1 1],'r');end

drawnow;disp('(paused)');pause;

%% Bayesian estimation for (c1,c2)
%disp('Computing joint Bayesian estimates for (c1,c2) for 600 image patches using anisotropic prior');
OL=0;
sw=L*[1 1]; % size of patch to be analyzed (sw <= image size)
dsw = OL*sw; % overlap parameter (if dsw = [0 0], no overlap)

% preprocessing
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(data,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters

% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for (c1,c2)
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20,ec1{2},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative priors GMRF(c2) and SAR(c1)
param_model.EST.GMRF.a=1.5*ones(size(param_model.EST.GMRF.a));
param_model.EST.GMRF.a=2*ones(size(param_model.EST.GMRF.a));
REGU=[1 1]; [ec2{3},Tc2,ec20tp,Tc20,ec1{3},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);

cc0=MASK(L/2:L:end,L/2:L:end);c1=zeros(size(cc0));c2=zeros(size(cc0)); for k=1:length(C1T);c1(cc0==k)=C1T(k);c2(cc0==k)=C2T(k);end;
figure(2);clf; set(gcf,'position',[10 10 500 500]);
subplot(221);imagesc(squeeze(c2));axis image; grid on; caxis(C2); title('c2');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec2{1}));axis image; grid on; caxis(C2); title('c2 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec2{2}));axis image; grid on; caxis(C2); title('c2 IG');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec2{3}));axis image; grid on; caxis(C2); title('c2 GMRF');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('c2 - Patch(x,y)');set(gca,'visible','off');set(h,'visible','on');

figure(3);clf; set(gcf,'position',[1000 10 500 500]);
subplot(221);imagesc(squeeze(c1));axis image; grid on; caxis(C1);title('c1');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec1{1}));axis image; grid on; caxis(C1); title('c1 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec1{2}));axis image; grid on; caxis(C1); title('c1 N');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec1{3}));axis image; grid on; caxis(C1); title('c1 SAR');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('c1 - Patch(x,y)');set(gca,'visible','off');set(h,'visible','on');



%% Bayesian estimation for (c1,c2) for OVERLAPPING patches
disp('(paused)'); pause; disp('Computing joint Bayesian estimates for (c1,c2) for 2166 OVERLAPPING (50%) image patches using anisotropic prior');
OL=0.5;
sw=L*[1 1]; % size of patch to be analyzed (sw <= image size)
dsw = OL*sw; % overlap parameter (if dsw = [0 0], no overlap)

% preprocessing
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(data,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters

% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for (c1,c2)
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20,ec1{2},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative priors GMRF(c2) and SAR(c1)
param_model.EST.GMRF.a=1.5*ones(size(param_model.EST.GMRF.a));

REGU=[1 1]; [ec2{3},Tc2,ec20tp,Tc20,ec1{3},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);

cc0=MASK(L/2:L/2:end-L/2,L/2:L/2:end-L/2);c1=zeros(size(cc0));c2=zeros(size(cc0)); for k=1:length(C1T);c1(cc0==k)=C1T(k);c2(cc0==k)=C2T(k);end;
figure(4);clf; set(gcf,'position',[500 10 500 500]);
%figure(2);clf; set(gcf,'position',[10 10 500 500]);
subplot(221);imagesc(squeeze(c2));axis image; grid on; caxis(C2); title('c2');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec2{1}));axis image; grid on; caxis(C2); title('c2 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec2{2}));axis image; grid on; caxis(C2); title('c2 IG');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec2{3}));axis image; grid on; caxis(C2); title('c2 GMRF');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('c2 - Patch(x,y)');set(gca,'visible','off');set(h,'visible','on');
figure(3);clf; set(gcf,'position',[1000 10 500 500]);
subplot(221);imagesc(squeeze(c1));axis image; grid on; caxis(C1);title('c1');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec1{1}));axis image; grid on; caxis(C1); title('c1 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec1{2}));axis image; grid on; caxis(C1); title('c1 N');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec1{3}));axis image; grid on; caxis(C1); title('c1 SAR');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('c1 - Patch(x,y)');set(gca,'visible','off');set(h,'visible','on');



%% Bayesian estimation for (c1,c2) for OVERLAPPING patches with MISSING DATA
disp('(paused)'); pause; disp('Computing joint Bayesian estimates for (c1,c2) for 2166 OVERLAPPING (50%) image patches using anisotropic prior');
OL=0.5;
sw=L*[1 1]; % size of patch to be analyzed (sw <= image size)
dsw = OL*sw; % overlap parameter (if dsw = [0 0], no overlap)
%
datanan=data; datanan(5*L+1:6*L,2*L+1:6*L)=nan;
figure(1);clf;set(gcf,'position',[10 500 500 400]);
imagesc(datanan);colormap bone;
title('A single image with two distinct zones of multifractality');
M2=MASK; M2(MASK==1)=C2T(1);M2(MASK==2)=C2T(2);M2(isnan(datanan))=nan;
figure(5);clf;hold on;set(gcf,'position',[600 500 500 400]);
imagesc(flipud(M2)); set(gca,'clim',C2); axis tight; set(gca,'ytick',[100:100:600]-60);set(gca,'yticklabel',[600:-100:100]);
title('Corresponding mask for values for c2');

% preprocessing
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(datanan,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters
% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for (c1,c2)
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20,ec1{2},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative priors GMRF(c2) and SAR(c1)
param_model.EST.GMRF.a=1.5*ones(size(param_model.EST.GMRF.a));
REGU=[1 1]; [ec2{3},Tc2,ec20tp,Tc20,ec1{3},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);

cc0=MASK(L/2:L/2:end-L/2,L/2:L/2:end-L/2);c1=zeros(size(cc0));c2=zeros(size(cc0)); for k=1:length(C1T);c1(cc0==k)=C1T(k);c2(cc0==k)=C2T(k);end;
figure(2);clf; set(gcf,'position',[10 10 500 500]);
subplot(221);imagesc(squeeze(c2));axis image; grid on; caxis(C2); title('c2');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec2{1}));axis image; grid on; caxis(C2); title('c2 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec2{2}));axis image; grid on; caxis(C2); title('c2 IG');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec2{3}));axis image; grid on; caxis(C2); title('c2 GMRF');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('c2 - Patch(x,y)');set(gca,'visible','off');set(h,'visible','on');
figure(3);clf; set(gcf,'position',[1000 10 500 500]);
subplot(221);imagesc(squeeze(c1));axis image; grid on; caxis(C1);title('c1');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec1{1}));axis image; grid on; caxis(C1); title('c1 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec1{2}));axis image; grid on; caxis(C1); title('c1 N');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec1{3}));axis image; grid on; caxis(C1); title('c1 SAR');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('c1 - Patch(x,y)');set(gca,'visible','off');set(h,'visible','on');

