%close all
clear all

% Load tools
currentpath = cd;
addpath([currentpath '/all_tools/']);


nC = 2;     % scenario
nanMask=0;  % simulate missing data

%% params
NII=1;     % number of integral scales (default: 1)
hpGMRF=[8 16 4]; % spatial / temporal / volume GMRF hyperparameters (default: [8 16 4])

%---you will most likely not need to touch the following parameters:
rmMean=0;   % use a mean-free model for c2 for joint estimation of (c1,c2) (default: 0)
nanIP=1;    % output missing data parameters sampled from prior (default: 1)
aSAR=1;     % estimate SAR hyperparameter (default: 1)
MEMSAVE=1;  % memory efficient computations, not storing chains (default: 1)

OL=0;       % overlap of analysis window (fraction of data cut size)
% sw=XXX;   % analysis window size, initialized below for different data sets

Nwt = 2;    % vanishing moments of wavelet
gamint = 0; % fractional integration parameter for wavelet leaders
% j1=X;     % scaling range, initialized below for different data sets
% j2=X;     % scaling range, initialized below for different data sets

Nbi = 300;  % burn-in of MCMC
Nmc = 700;  % length of MCMC


% params for this demo script only
doc1 = 0; % 1: jointly estimate (c1,c2). 0: only estimate c2
tag=[]; % initialized here to avoid error when not specified

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SYNTHETIC DATA SETS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch nC
    case 0 % single TS
        load('demo_TS_2.mat');l=1;k = 3;m=5; data = squeeze(data(l,m,k,:))';
        c2 = c2(l,m,k);c1 = c1(l,m,k); C1 = [0.6 0.8]; C2 = [-0.09 0]; % for figure
        sw = 2^9; % length of time series to be analyzed (sw <= L)
        dsw = OL*sw; % overlap parameter (if dsw = 0, no overlap)
        j1=2; j2=4;
        c1=-0.5*ones(size(data)); c2=zeros(size(data));
    case 1 % 1D (line) TS
        load('demo_TS_2.mat'); l=1;k = 3; data = squeeze(data(l,:,k,:));
        if nanMask;  data(2:floor(end/2)-1,:)=nan;data(10:11,:)=nan; end
        c2 = c2(l,:,k);c1 = c1(l,:,k); C1 = [0.6 0.8]; C2 = [-0.09 0]; % for figure
        c1(isnan(data(:,1)))=nan;c2(isnan(data(:,1)))=nan;
        sw = 2^11; % length of time series to be analyzed (sw <= L)
        dsw = OL*sw; % overlap parameter (if dsw = 0, no overlap)
        j1=3; j2=7;
    case 2 % 2D (plane) TS
        load('demo_TS_2.mat'); k = 3; data = squeeze(data(:,:,k,:));
        if nanMask; data(:,floor(1:end/4),:)=nan; data(floor(1:end/3),:,:)=nan; data(:,floor(end/2)+[0:1],:)=nan;data(1,:,:)=nan;data(end,:,:)=nan; end
        c2 = c2(:,:,k);c1 = c1(:,:,k); C1 = [0.6 0.8]; C2 = [-0.09 0]; % for figure
        c1(isnan(data(:,:,1)))=nan;c2(isnan(data(:,:,1)))=nan;
        sw = 2^11; % length of time series to be analyzed (sw <= L)
        dsw = OL*sw; % overlap parameter (if dsw = 0, no overlap)
        j1=3; j2=7;
    case 3; % 3D (volume) TS
        load('demo_TS_2.mat'); data=data(1:2:end,1:2:end,1:end,1:2:end);
        c2 = c2(1:2:end,1:2:end,1:end);c1 = c1(1:2:end,1:2:end,1:end); C1 = [0.6 0.8]; C2 = [-0.09 0]; % for figure
        if nanMask; data(1:floor(1:end/3),floor(1:end/3),:,:)=nan; data(floor(end/2),floor(end/2)+[0 1],:,:)=nan; end
        c1(isnan(data(:,:,:,1)))=nan;c2(isnan(data(:,:,:,1)))=nan;
        sw = 2^10; % length of time series to be analyzed (sw <= L)
        dsw = OL*sw; % overlap parameter (if dsw = 0, no overlap)
        j1=2; j2=5;
        %tag ='3D'; %
        tag ='2D+1D'; %
    case 4  % Single image
        load('MRWseries640_6.mat'); data=squeeze(data(:,:,3));
        L = 2^6; 
        sw=L*[1 1]; % size of patch to be analyzed (sw <= image size)
        dsw = OL*sw; % overlap parameter (if dsw = [0 0], no overlap)
        cc0=MASK(L/2:L:end,L/2:L:end);c1=zeros(size(cc0));c2=zeros(size(cc0));for k=1:length(C1);c1(cc0==k)=C1(k);c2(cc0==k)=C2(k);end
        C1 = [0.5 0.9]; C2 = [-0.05 0]; % for figure
        if nanMask data(L*floor(u(1)/2)+[1:2*L],L*floor(u(2)/2)+[1:L]+L/2,1)=nan; end
        j1 =1; j2=2;
    case 5  % Image sequence
        load('MRWseries640_6.mat');
        L = 2^6;
        sw=L*[1 1]; % size of patch to be analyzed (sw <= image size)
        dsw = OL*sw; % overlap parameter (if dsw = [0 0], no overlap)
        cc0=MASK(L/2:L:end,L/2:L:end);c1=zeros(size(cc0));c2=zeros(size(cc0)); for k=1:length(C1);c1(cc0==k)=C1(k);c2(cc0==k)=C2(k);end; c1=repmat(c1,[1 1 6]);c2=repmat(c2,[1 1 6]);
        C1 = [0.5 0.9]; C2 = [-0.05 0]; % for figure
        if nanMask; data(L*floor(u(1)/2)+[1:2*L],L*floor(u(2)/2)+[1:L]+L/2,1,:)=nan; end
        j1 =1; j2=2;
        %tag ='3D';
        tag ='2D+1D'; %
end

tic

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PREPROCESSING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Data cutting and computation of DWT and DFT
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(data,sw,dsw,j1,j2,Nwt,gamint);
% Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters
[param_model] = init_model_mvmfa(param_data,'NIScale',NII);
    % can replace default params using name-value pairs or do it manually (see below)
    [param_model2] = init_model_mvmfa(param_data, 'Nbi', Nbi, 'Nmc', Nmc, 'hpGMRF', hpGMRF, 'rmMean', rmMean,'nanIP',nanIP,'MEMSAVE',MEMSAVE,'aSAR',aSAR);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ESTIMATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%***** Linear regression (if wanted) *****
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,1);

%****** Bayesian estimation ******

%--- Can update/override default parameters if needed
pL.Nbi = Nbi; pL.Nmc = Nmc;
pL.EST.rmMean=rmMean;       % use a mean-free model for c2 (default: 1)
pL.EST.nanIP=nanIP;         % draw missing data parameters from prior (default: 1)
pL.EST.MEMSAVE=MEMSAVE;     % memory efficient computations, not storing chains (default: 1)
pL.EST.epi_SAMP=aSAR;       % estimate SAR hyperparameter (default: estimated)
%
pL.GMRF.a = hpGMRF(1)*ones(1,4); % Spatial
pL.GMRF.b = hpGMRF(2)*ones(1,2); % Temporal
pL.GMRF.c = hpGMRF(3)*ones(1,8); % 3D isotropic
%-------


if doc1 % Joint estimation of (c1,c2)
    % Non informative priors for c1 and c2
    REGU=[0 0];
    [ec2{2},Tc2,ec20,Tc20,ec1{2},Tc1] = est_bayes_mvmfa(lxft,param_model,REGU);
    % Informative priors SAR(c1) and GMRF(c2)
    REGU=[1 1];
    [ec2{3},Tc2,ec20,Tc20,ec1{3},Tc1,ec10,Tc10,epc1,Tepc1] = est_bayes_mvmfa(lxft,param_model,REGU,tag);
    % Informative prior GMRF(c2) and Non informative priors for c1
    % REGU=[1 0]; [ec2{4},Tc2,ec20,Tc20,ec1{4},Tc1,ec10,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU,tag);
else % Only estimation of c2
    % Non informative prior for c2
    REGU=[0 0];
    [ec2{2},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU);
    % Informative prior GMRF(c2)
    REGU=[1 0];
    [ec2{3},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU,tag);
end

% correct c1 for fractional integration
for i=1:3, try ec1{i}=ec1{i}-gamint; end; end 
toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DISPLAY ONLY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

try
    for i = 1:3
        ERR{1,2} = 'LF';ERR{1,3} = 'IG';ERR{1,4} = 'GMRF';
        if doc1; ERR{1,3} = 'IG/N';ERR{1,4} = 'GMRF/SAR';end
        ERR{2,i+1} = nanmean((c2(:)-ec2{i}(:)).^2); ERR{2,1}='c2';
        try; ERR{3,i+1} = nanmean((c1(:)-ec1{i}(:)).^2); ERR{3,1}='c1';end
    end
    disp('            ------ AVERAGE ROOT SQUARE ERROR -------');
    disp(ERR);
end
try
    ec2{2};
    figure(1);clf
    subplot(221);imagesc(squeeze(c2(:,:,1)));axis image; grid on; caxis(C2); title('c2')
    subplot(222);imagesc(squeeze(ec2{1}(:,:,1)));axis image; grid on; caxis(C2); title('c2 LF')
    subplot(223);imagesc(squeeze(ec2{2}(:,:,1)));axis image; grid on; caxis(C2); title('c2 IG')
    subplot(224);imagesc(squeeze(ec2{3}(:,:,1)));axis image; grid on; caxis(C2); title('c2 GMRF')
end
try
    ec1{2};
    figure(2);clf
    subplot(221);imagesc(squeeze(c1(:,:,1)));axis image; grid on; caxis(C1);title('c1')
    subplot(222);imagesc(squeeze(ec1{1}(:,:,1)));axis image; grid on; caxis(C1); title('c1 LF')
    subplot(223);imagesc(squeeze(ec1{2}(:,:,1)));axis image; grid on; caxis(C1); title('c1 N')
    subplot(224);imagesc(squeeze(ec1{3}(:,:,1)));axis image; grid on; caxis(C1); title('c1 SAR')
end

