%% demo_TS_1Dline.m
% tutorial for joint estimation of c2 for a collection of time series
% registered on, e.g., a linear 1D sensor array 

clear all; clc;

% Load tools
pth=which('est_bayes_mvmfa');if ispc; ii=strfind(pth,'\');else;ii=strfind(pth,'/');end;pth=pth(1:ii(end));addpath(genpath(pth));
%currentpath = cd;addpath([currentpath(1:end-5)]);addpath([currentpath(1:end-5) '/all_tools/']);

%% params
NII=1;     % number of integral scales (default: 1)
Nwt = 2;    % vanishing moments of wavelet
gamint = 0; % fractional integration parameter for wavelet leaders
j1=3;       % scaling range, initialized below for different data sets
j2=7;       % scaling range, initialized below for different data sets

Nbi = 300;  % burn-in of MCMC
Nmc = 1000;  % length of MCMC

load('demo_TS_2.mat');l=2;k = 3; data = squeeze(data(l,:,k,:));
c2 = c2(l,:,k);c1 = c1(l,:,k); C1 = [0.6 0.8]; C2 = [-0.09 0]; % for figure
c1(isnan(data(:,1)))=nan;c2(isnan(data(:,1)))=nan;
zdata=zscore(data')/4;
figure(1);clf;hold on;set(gcf,'position',[10 500 500 400]);
for k=1:size(data,1);LS{k}=num2str(k);plot(zdata(:,k)+k-zdata(1,k));end
grid on;set(gca,'xlim',[1 2^11]);title('The data set: ensemble of 15 time series');ylabel('time series');%legend(LS);
set(gca,'ytick',[1:15]);set(gca,'yticklabel',[1:15]);



%% Bayesian estimation for c2
disp('The data set'); disp('(paused)'); pause; disp('Computing Bayesian estimates for c2');
OL=0;       % overlap of analysis window (fraction of data cut size)
sw=2^11;     % analysis window size (sw < length(TS))
dsw = OL*sw; % overlap parameter (if dsw = 0, no overlap)

% preprocessing
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(data,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters
% param_model.EST.Nbi=300;
% param_model.EST.Nmc=1000;
% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for c2
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative prior GMRF(c2)
param_model.EST.GMRF.b=[20 20];
REGU=[1 0]; [ec2{3},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU);
%%
figure(2);clf;tt=1:length(squeeze(c2(:,:,1)));set(gcf,'position',[10 100 500 300]);
plot(tt,squeeze(c2(:,:,1)),'r--',tt,squeeze(ec2{1}(:,:,1)),'mx-',tt,squeeze(ec2{2}(:,:,1)),'ko-',tt,squeeze(ec2{3}(:,:,1)),'bd-');grid on;
title('Estimates for c2');legend('theo','LF','IG','GMRF');xlabel('time series')



%% change regularization parameter
disp('(paused)'); pause; disp('Change regularization parameter for GMRF prior');
% Informative prior GMRF(c2)
param_model.EST.GMRF.b=[100 100];
REGU=[1 0]; [ec2{4},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU);

figure(2);clf;tt=1:length(squeeze(c2(:,:,1)));
plot(tt,squeeze(c2(:,:,1)),'r--',tt,squeeze(ec2{1}(:,:,1)),'mx-',tt,squeeze(ec2{2}(:,:,1)),'ko-',tt,squeeze(ec2{3}(:,:,1)),'bd-',tt,squeeze(ec2{4}(:,:,1)),'bs-');grid on;
title('Estimates for c2');legend('theo','LF','IG','GMRF a10','GMRF a50'); xlabel('time series')



%% visualize MCMC chains
disp('(paused)'); pause; disp('Visualize Markov chains to check convergence');
param_model.EST.MEMSAVE=0; % store and output monte carlo chains
REGU=[1 0]; [ec2,Tc2] = est_bayes_mvmfa(lxft,param_model,REGU);

figure(3); clf; plot(Tc2'); grid on; set(gcf,'position',[600 100 500 300]);
ylabel('c2'); title('Markov chains');legend(LS);



%% localize estimation in time
disp('(paused)'); pause; disp('Localize estimation in time by using windows');
OL=0;       % overlap of analysis window (fraction of data cut size): no overlap
sw=2^8;     % analysis window size 1/8 of time series length
dsw = OL*sw; % overlap parameter (if dsw = 0, no overlap)
j2=5;
clear ec2* ec1*
% preprocessing
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(data,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters

% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for c2
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative prior GMRF(c2)
param_model.EST.GMRF.b=[10 10];
REGU=[1 0]; [ec2{3},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU);

figure(1);hold on;yl=get(gca,'ylim'); hold on; for l=1:7, plot([1 1]*sw*l,yl,'k'); end;hold off;
xlabel('windows');
figure(4);clf;set(gcf,'position',[600 500 500 400]);
    subplot(221);imagesc(repmat(c2',1,8));axis image; grid on; caxis(C2); title('c2');xlabel('window');ylabel('time series');
    subplot(222);imagesc(squeeze(ec2{1}(:,:,1)));axis image; grid on; caxis(C2); title('c2 LF');xlabel('window');ylabel('time series');
    subplot(223);imagesc(squeeze(ec2{2}(:,:,1)));axis image; grid on; caxis(C2); title('c2 IG');xlabel('window');ylabel('time series');
    subplot(224);imagesc(squeeze(ec2{3}(:,:,1)));axis image; grid on; caxis(C2); title('c2 GMRF');xlabel('window');ylabel('time series');
    axes('Units','Normal');h = title('Time windowed estimation');set(gca,'visible','off');set(h,'visible','on');
    
    
%% further localize estimation in time with overlapping windows
disp('(paused)'); pause; disp('Further localize estimation in time by using overlapping windows');
OL=0.9;       % overlap of analysis window (fraction of data cut size): no overlap
sw=2^8;     % analysis window size 1/8 of time series length
dsw = sw-1;round(OL*sw); % overlap parameter (if dsw = 0, no overlap)
j2=5;
clear ec2* ec1*
% preprocessing
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(data,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters
param_model.EST.Nbi=300;param_model.EST.Nmc=1000;

% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for c2
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative prior GMRF(c2)
param_model.EST.GMRF.b=[10 10];
REGU=[1 0]; [ec2{3},Tc2,ec20tp,Tc20] = est_bayes_mvmfa(lxft,param_model,REGU);

figure(1);hold on;yl=get(gca,'ylim'); hold on; for l=1:15, plot([1 1]*sw/2*l,yl,'k'); end;hold off;
xlabel('window centers');

%
figure(4);clf
    subplot(221);imagesc(repmat(c2',1,15));axis image; grid on; caxis(C2); title('c2');xlabel('window');ylabel('time series');
    subplot(222);imagesc(squeeze(ec2{1}(:,:,1)));axis image; grid on; caxis(C2); title('c2 LF');xlabel('window');ylabel('time series');
    subplot(223);imagesc(squeeze(ec2{2}(:,:,1)));axis image; grid on; caxis(C2); title('c2 IG');xlabel('window');ylabel('time series');
    subplot(224);imagesc(squeeze(ec2{3}(:,:,1)));axis image; grid on; caxis(C2); title('c2 GMRF');xlabel('window');ylabel('time series');
    axes('Units','Normal');h = title('Overlapping time windows');set(gca,'visible','off');set(h,'visible','on');
    
