%% demo_TS_2Dplane.m
% tutorial for the joint estimation of (c1,c2) for a collection of time series
% registered on a planar array of sensors

clear all; clc;

% Load tools
pth=which('est_bayes_mvmfa');if ispc; ii=strfind(pth,'\');else;ii=strfind(pth,'/');end;pth=pth(1:ii(end));addpath(genpath(pth));
%currentpath = cd;addpath([currentpath(1:end-5)]);addpath([currentpath(1:end-5) '/all_tools/']);

%% params
NII=1;     % number of integral scales (default: 1)
Nwt = 2;    % vanishing moments of wavelet
gamint = 0; % fractional integration parameter for wavelet leaders
j1=3;       % scaling range, initialized below for different data sets
j2=7;       % scaling range, initialized below for different data sets

Nbi = 300;  % burn-in of MCMC
Nmc = 1000;  % length of MCMC


%% syntehtic data set
load('demo_TS_2.mat'); k = 2; data = squeeze(data(:,:,k,:));
c2 = c2(:,:,k);c1 = c1(:,:,k); C1 = [0.6 0.8]; C2 = [-0.09 0]; % for figure
c1(isnan(data(:,:,1)))=nan;c2(isnan(data(:,:,1)))=nan;
zdata=zscore(data,0,3)/4;

figure(1);clf;hold on;set(gcf,'position',[10 500 500 400]);
for k1=1:size(data,1);
    for k2=1:size(data,2);
        plot3(squeeze(zdata(k1,k2,:))+k1-squeeze(zdata(k1,k2,1)), k2+zeros(size(squeeze(zdata(k1,k2,:)))), 1:length(squeeze(zdata(k1,k2,:))));
    end
end
title('The data set: ensemble of 14 x 15 time series');zlabel('time');xlabel('y');ylabel('x');
grid on;
set(gca,'zlim',[1 2^11]);
axis([0 15 0 16 1 2^11]);
set(gca,'xtick',[1:14]);set(gca,'xticklabel',[1:14]);
set(gca,'ytick',[1:15]);set(gca,'yticklabel',[1:15]);
view([-26,36])
disp('The data set...'); drawnow;pause(0.1);

%% Bayesian estimation for (c1,c2)
%disp('Computing joint Bayesian estimates for (c1,c2) for 210 time series');
OL=0;       % overlap of analysis window (fraction of data cut size)
sw=2^11;     % analysis window size (sw < length(TS))
dsw = OL*sw; % overlap parameter (if dsw = 0, no overlap)

% preprocessing
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(data,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters

% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for (c1,c2)
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20,ec1{2},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative priors GMRF(c2) and SAR(c1)
param_model.EST.GMRF.a=2*ones(size(param_model.EST.GMRF.a));
REGU=[1 1]; [ec2{3},Tc2,ec20tp,Tc20,ec1{3},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);

figure(2);clf; set(gcf,'position',[10 10 500 500]);
subplot(221);imagesc(squeeze(c2(:,:,1)));axis image; grid on; caxis(C2); title('c2');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec2{1}(:,:,1)));axis image; grid on; caxis(C2); title('c2 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec2{2}(:,:,1)));axis image; grid on; caxis(C2); title('c2 IG');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec2{3}(:,:,1)));axis image; grid on; caxis(C2); title('c2 GMRF a=2');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('Estimates for c2');set(gca,'visible','off');set(h,'visible','on');

figure(3);clf; set(gcf,'position',[1000 10 500 500]);
subplot(221);imagesc(squeeze(c1(:,:,1)));axis image; grid on; caxis(C1);title('c1');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec1{1}(:,:,1)));axis image; grid on; caxis(C1); title('c1 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec1{2}(:,:,1)));axis image; grid on; caxis(C1); title('c1 N');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec1{3}(:,:,1)));axis image; grid on; caxis(C1); title('c1 SAR');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('Estimates for c1');set(gca,'visible','off');set(h,'visible','on');

break;

%% change regularization parameter
disp('(paused)'); pause; disp('Change regularization parameter for GMRF prior');
% Informative prior GMRF(c2)
param_model.EST.GMRF.a=10*ones(size(param_model.EST.GMRF.a));
REGU=[1 1]; [ec2{3},Tc2,ec20tp,Tc20,ec1{3},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);

%%
figure(4);clf; set(gcf,'position',[500 10 500 500]);
subplot(221);imagesc(squeeze(c2(:,:,1)));axis image; grid on; caxis(C2); title('c2');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec2{3}(:,:,1)));axis image; grid on; caxis(C2); title('c2 GMRF a=10');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(c1(:,:,1)));axis image; grid on; caxis(C1); title('c1');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec1{3}(:,:,1)));axis image; grid on; caxis(C1); title('c1 SAR');xlabel('x');ylabel('y');


%% Simulate missing data
disp('(paused)'); pause; disp('Estimation with missing data');
% preprocessing
Pct_missing=20; % percentage of missing data
nn=size(data); mm=rand(nn(1:2));mm(mm>=Pct_missing/100)=1;mm(mm<Pct_missing/100)=nan;datanan=data;
for k1=1:nn(1);for k2=1:nn(2);datanan(k1,k2,end/4:end/2)=datanan(k1,k2,end/4:end/2)*mm(k1,k2);end;end
[lxft,param_data] = preproc_data_mvmfa(datanan,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters
param_model.EST.GMRF.a=10*ones(size(param_model.EST.GMRF.a));

% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for (c1,c2)
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20,ec1{2},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative priors GMRF(c2) and SAR(c1)
REGU=[1 1]; [ec2{3},Tc2,ec20tp,Tc20,ec1{3},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);

figure(2);clf; set(gcf,'position',[10 10 500 500]);
subplot(221);imagesc(mm.*squeeze(c2(:,:,1)));axis image; grid on; caxis(C2); title('c2');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec2{1}(:,:,1)));axis image; grid on; caxis(C2); title('c2 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec2{2}(:,:,1)));axis image; grid on; caxis(C2); title('c2 IG');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec2{3}(:,:,1)));axis image; grid on; caxis(C2); title('c2 GMRF a=10');xlabel('x');ylabel('y');
axes('Units','Normal');h = title(['Estimates for c2: ',num2str(Pct_missing),'% missing data']);set(gca,'visible','off');set(h,'visible','on');

figure(3);clf; set(gcf,'position',[1000 10 500 500]);
subplot(221);imagesc(mm.*squeeze(c1(:,:,1)));axis image; grid on; caxis(C1);title('c1');xlabel('x');ylabel('y');
subplot(222);imagesc(squeeze(ec1{1}(:,:,1)));axis image; grid on; caxis(C1); title('c1 LF');xlabel('x');ylabel('y');
subplot(223);imagesc(squeeze(ec1{2}(:,:,1)));axis image; grid on; caxis(C1); title('c1 N');xlabel('x');ylabel('y');
subplot(224);imagesc(squeeze(ec1{3}(:,:,1)));axis image; grid on; caxis(C1); title('c1 SAR');xlabel('x');ylabel('y');
axes('Units','Normal');h = title(['Estimates for c1: ',num2str(Pct_missing),'% missing data']);set(gca,'visible','off');set(h,'visible','on');

