%% demo_TS_2Dplane.m
% tutorial for the joint estimation of (c1,c2) for a collection of time series
% registered on a planar array of sensors

clear all; clc;

% Load tools
pth=which('est_bayes_mvmfa');if ispc; ii=strfind(pth,'\');else;ii=strfind(pth,'/');end;pth=pth(1:ii(end));addpath(genpath(pth));
%currentpath = cd;addpath([currentpath(1:end-5)]);addpath([currentpath(1:end-5) '/all_tools/']);

%% params
NII=1;     % number of integral scales (default: 1)
Nwt = 2;    % vanishing moments of wavelet
gamint = 0; % fractional integration parameter for wavelet leaders
j1=3;       % scaling range, initialized below for different data sets
j2=7;       % scaling range, initialized below for different data sets

Nbi = 300;  % burn-in of MCMC
Nmc = 1000;  % length of MCMC


%% SYNTHETIC DATA SET
load('demo_TS_2.mat'); k = 2; 
C1 = [0.6 0.8]; C2 = [-0.09 0]; % for figure
zdata=zscore(squeeze(data(:,:,k,:)),0,3)/4;

figure(1);clf;hold on;set(gcf,'position',[10 500 500 400]);
for k1=1:size(data,1);
    for k2=1:size(data,2);
        plot3(squeeze(zdata(k1,k2,:))+k1-squeeze(zdata(k1,k2,1)), k2+zeros(size(squeeze(zdata(k1,k2,:)))), 1:length(squeeze(zdata(k1,k2,:))));
    end
end
title(['Slice z=',num2str(k),': 14 x 15 time series out of the volume of 14 x 15 x 6 time series']);zlabel('time');xlabel('y');ylabel('x');
set(gca,'zlim',[1 2^11]);axis([0 15 0 16 1 2^11]);grid on;view([-26,36])
set(gca,'xtick',[1:14]);set(gca,'xticklabel',[1:14]);set(gca,'ytick',[1:15]);set(gca,'yticklabel',[1:15]);

figure(11);clf; set(gcf,'position',[550 500 700 400]);
subplot(231);imagesc(squeeze(c2(:,:,2)));axis image; grid on; caxis(C2); title('c2 slice z=2');xlabel('x');ylabel('y');
subplot(234);imagesc(squeeze(c1(:,:,2)));axis image; grid on; caxis(C1); title('c1 slice z=2');xlabel('x');ylabel('y');
subplot(232);imagesc(squeeze(c2(7,:,:))');axis image; grid on; caxis(C2); title('c2 slice y=7');xlabel('x');ylabel('z');
subplot(235);imagesc(squeeze(c1(7,:,:))');axis image; grid on; caxis(C1); title('c1 slice y=7');xlabel('x');ylabel('z');
subplot(233);imagesc(squeeze(c2(:,6,:)));axis image; grid on; caxis(C2); title('c2 slice x=6');xlabel('z');ylabel('y');
subplot(236);imagesc(squeeze(c1(:,6,:)));axis image; grid on; caxis(C1); title('c1 slice x=6');xlabel('z');ylabel('y');
disp('The data set...'); drawnow;pause(0.1);


%% Bayesian estimation for (c1,c2) using isotropic (3D) prior
disp('Computing joint Bayesian estimates for (c1,c2) for volume of 14 x 15 x 6 =  1260 time series using isotropic prior');
OL=0;
sw=2^11; % size of patch to be analyzed (sw <= time series length)
dsw = OL*sw; % overlap parameter (if dsw = [0 0], no overlap)
tag ='3D'; % use anisotropic (time-space) prior


% preprocessing
[lxft,param_data,data0,pdata] = preproc_data_mvmfa(data,sw,dsw,j1,j2,Nwt,gamint); % Data cutting and computation of DWT and DFT
[param_model] = init_model_mvmfa(param_data,'NIScale',NII,'Nbi',Nbi,'Nmc',Nmc); % Computation of fixed terms of the spectral density, reparametrization + initialization of prior parameters
% linear fit (for comparison)
[ec2{1},ec20{1},ec1{1},ec10{1}] = linfit_cp(param_data,0);
% Non informative prior for (c1,c2)
REGU=[0 0]; [ec2{2},Tc2,ec20tp,Tc20,ec1{2},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU);
% Informative prior GMRF(c2)
param_model.EST.GMRF.c=5*ones(size(param_model.EST.GMRF.c));
REGU=[1 1]; [ec2{3},Tc2,ec20tp,Tc20,ec1{3},Tc1,ec10tp,Tc10] = est_bayes_mvmfa(lxft,param_model,REGU,tag);


figure(2);clf; set(gcf,'position',[10 320 750 200]);
subplot(141);imagesc(squeeze(c2(:,:,2)));axis image; grid on; caxis(C2); title('c2');xlabel('x');ylabel('y');
subplot(142);imagesc(squeeze(ec2{1}(:,:,2)));axis image; grid on; caxis(C2); title('c2 LF');xlabel('x');ylabel('y');
subplot(143);imagesc(squeeze(ec2{2}(:,:,2)));axis image; grid on; caxis(C2); title('c2 IG');xlabel('x');ylabel('y');
subplot(144);imagesc(squeeze(ec2{3}(:,:,2)));axis image; grid on; caxis(C2); title('c2 GMRF 3D');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('Slice z = 2');set(gca,'visible','off');set(h,'visible','on');

figure(4);clf; set(gcf,'position',[750 320 750 200]);
subplot(141);imagesc(squeeze(c1(:,:,2)));axis image; grid on; caxis(C1); title('c1');xlabel('x');ylabel('y');
subplot(142);imagesc(squeeze(ec1{1}(:,:,2)));axis image; grid on; caxis(C1); title('c1 LF');xlabel('x');ylabel('y');
subplot(143);imagesc(squeeze(ec1{2}(:,:,2)));axis image; grid on; caxis(C1); title('c1 IG');xlabel('x');ylabel('y');
subplot(144);imagesc(squeeze(ec1{3}(:,:,2)));axis image; grid on; caxis(C1); title('c1 GMRF 3D');xlabel('x');ylabel('y');
axes('Units','Normal');h = title('Slice z = 2');set(gca,'visible','off');set(h,'visible','on');

figure(12);clf; set(gcf,'position',[10 -15 350 180]);
subplot(141);imagesc(squeeze(c2(:,6,:)));axis image; grid on; caxis(C2); title('c2');xlabel('z');ylabel('y');
subplot(142);imagesc(squeeze(ec2{1}(:,6,:)));axis image; grid on; caxis(C2); title('c2 LF');xlabel('z');ylabel('y');
subplot(143);imagesc(squeeze(ec2{2}(:,6,:)));axis image; grid on; caxis(C2); title('c2 IG');xlabel('z');ylabel('y');
subplot(144);imagesc(squeeze(ec2{3}(:,6,:)));axis image; grid on; caxis(C2); title('c2 GMRF 3D');xlabel('z');ylabel('y');
axes('Units','Normal');h = title('Slice x = 6');set(gca,'visible','off');set(h,'visible','on');

figure(14);clf; set(gcf,'position',[750 -15 350 180]);
subplot(141);imagesc(squeeze(c1(:,6,:)));axis image; grid on; caxis(C1); title('c1');xlabel('z');ylabel('y');
subplot(142);imagesc(squeeze(ec1{1}(:,6,:)));axis image; grid on; caxis(C1); title('c1 LF');xlabel('z');ylabel('y');
subplot(143);imagesc(squeeze(ec1{2}(:,6,:)));axis image; grid on; caxis(C1); title('c1 IG');xlabel('z');ylabel('y');
subplot(144);imagesc(squeeze(ec1{3}(:,6,:)));axis image; grid on; caxis(C1); title('c1 GMRF 3D');xlabel('z');ylabel('y');
axes('Units','Normal');h = title('Slice x = 6');set(gca,'visible','off');set(h,'visible','on');

figure(22);clf; set(gcf,'position',[10 170 750 100]);
subplot(141);imagesc(squeeze(c2(7,:,:))');axis image; grid on; caxis(C2); title('c2');xlabel('x');ylabel('z');
subplot(142);imagesc(squeeze(ec2{1}(7,:,:))');axis image; grid on; caxis(C2); title('c2 LF');xlabel('x');ylabel('z');
subplot(143);imagesc(squeeze(ec2{2}(7,:,:))');axis image; grid on; caxis(C2); title('c2 IG');xlabel('x');ylabel('z');
subplot(144);imagesc(squeeze(ec2{3}(7,:,:))');axis image; grid on; caxis(C2); title('c2 GMRF 3D');xlabel('x');ylabel('z');
axes('Units','Normal');h = title('Slice y = 7');set(gca,'visible','off');set(h,'visible','on');

figure(24);clf; set(gcf,'position',[750 170 750 100]);
subplot(141);imagesc(squeeze(c1(7,:,:))');axis image; grid on; caxis(C1); title('c1');xlabel('x');ylabel('z');
subplot(142);imagesc(squeeze(ec1{1}(7,:,:))');axis image; grid on; caxis(C1); title('c1 LF');xlabel('x');ylabel('z');
subplot(143);imagesc(squeeze(ec1{2}(7,:,:))');axis image; grid on; caxis(C1); title('c1 IG');xlabel('x');ylabel('z');
subplot(144);imagesc(squeeze(ec1{3}(7,:,:))');axis image; grid on; caxis(C1); title('c1 GMRF 3D');xlabel('x');ylabel('z');
axes('Units','Normal');h = title('Slice y = 7');set(gca,'visible','off');set(h,'visible','on');
