function [param_model] = init_model_mvmfa(param_data,varargin)
%
% function [param_model] = init_model_mvmfa(param_data,OPTIONAL_PARAMS)
%
% Computation and initialization of parameters of Bayesian model.
%
% INPUT
% - param_data  :   structure with parameters produced by "preproc_data_mvmfa.m"
% - OPTIONAL_PARAMS: optional parameters given by the following name-value pairs
%       - 'NIScale' : N+ - number of integral scales of the data (default: 1)
%       - 'Nbi'     : N+ - number of burn-in samples of MCM chains (default: 500)
%       - 'Nmc'     : N+ - total number of samples of MCM chains (default: 800)
%       - 'hpGMRF'  : [R+ R+ R+] - spatial / temporal / volume GMRF hyperparameters (default: [8 16 4])
%       - 'MEMSAVE' : (0 or 1) - 1: low-memory computation not storing MCM chains (default: 1)
%       - 'nanIP'   : (0 or 1) - 1: output missing data parameters sampled from prior (default: 1)
%       - 'rmMean'  : [ADVANCED] (0 or 1) - 1: use a mean-free model for c2 for joint estimation of (c1,c2) (default: 0)
%       - 'aSAR'    : [ADVANCED] (0 or 1) - 1: estimate SAR hyperparameter (default: 1)
%                        NOTE: if you choose NOT to estimate the SAR hyperparameters, you can manually set it in param_model.param.epi
%       - 'PostVal' : [ADVANCED] (0 or 1) - 1: output posterior value and MAP estimate for univariate / noninformative priors (default: 0)
% OUTPUT
% - param_model :   structure with all parameters of the Bayesian model, and for estimation
%        The field "param_model.EST" contains all the OPTIONAL_PARAMS described above:
%        They can be manually changed/updated a posteriori by the user. 
%
% HW, BCN&TLS, 03/2017

valstrs={'NIScale'};
vnames={'NII'};
strs=varargin(1:2:end);vals=varargin(2:2:end);
if ~isempty(vals)
    for k=1:length(strs)
        id=find(strcmp(valstrs,strs{k}));
        if ~isempty(id);
            eval([vnames{id},'=vals{k};']);
        end
    end
end

try if isempty(NII); NII=1; end; catch; NII=1; end

% Computation of fixed terms of the spectral density
[LLc,ind0] = get_LLc(param_data.N,param_data.j1,param_data.j2,param_data.Nwt,NII);
% Reparametrization + initialization of prior parameters
U.u=param_data.u; U.dim=param_data.dim;
[param_model] = pLLc_prior(LLc,param_data.j1,param_data.j2,param_data.Nj,ind0,U,varargin); 
param_model.NII=NII;
param_model.N=param_data.N;